
# Fast Compressed Segmentation Volumes for Scientific Visualization

The directory contains exemplary code for the compression method proposed in "Fast Compressed Segmentation Volumes for Scientific Visualization".
The project is kept minimal to allow integration into other systems.
For details regarding the compression method, we refer to the paper.

This directory contains:
* C++ code for compressing and decompressing segmentation volumes on the CPU,
* simple tests for verifying results on multiple level-of-details,
* code to generate example 3D-Voronoi volumes, and
* GLSL code for implementing GPU based decompression.

## Dependencies
This project only depends on GLM which can be installed on Debian / Ubuntu systems with `sudo apt install libglm-dev`.
To build the project, you need CMake which can be installed with `sudo apt install cmake`. 
If your compiler supports it, OpenMP is used to parallelize (de)compression and example data generation.

To build the CMake project, move to the root directory and execute

```
mkdir cmake-build-release && cd cmake-build-release
cmake -DCMAKE_BUILD_TYPE=Release ..
make -j 8
```
followed by `./csvol` to run the example program.

## Example Data
The example program creates a Voronoi-like data set by sampling a point set and computing a nearest neighbor within this point set for each voxel.
Note that this code is not optimized and will lead to long runtimes for high volume dimensions or label densities.
We highly recommend running the code as a Release build.
Once the example data is computed, it is exported to a local file which will be re-imported when the program runs for the next time.  
If you want to use your own data from hdf5 files, we recommend [HighFive](https://github.com/BlueBrain/HighFive) which can easily be integrated as a CMake subdirectory.


## Project Structure

| Directory / Files                | Functionality                                                                    |
|----------------------------------|----------------------------------------------------------------------------------|
| main.cpp                         | contains the example program                                                     |
| shader                           | GLSL code for decompressing Compressed Segmentation Volumes on the GPU           |
| include/rANS                     | contains a modified version of the rANS implementation released by Fabian Giesen |
| CompressedSegmentationVolume.hpp | class containing the main compression and decompression implementation           |
| VolumeCompressionBase.hpp        | contains the reference multi grid construction                                   |
| Volume.hpp                       | contains code to generate an example Voronoi volume and import/export functions  |

## Contact

Max Piochowiak ([max.piochowiak@kit.edu](mailto:max.piochowiak@kit.edu)), Karlsruhe Institute of Technology, Institute for Visualization and Data Analysis, Germany


## License
Copyright(c) 2023 Max Piochowiak

<a rel="license" href="http://creativecommons.org/licenses/by-nc/4.0/"><img alt="Creative Commons License" style="border-width:0" src="https://i.creativecommons.org/l/by-nc/4.0/88x31.png" /></a><br />This work is licensed under a <a rel="license" href="http://creativecommons.org/licenses/by-nc/4.0/">Creative Commons Attribution-NonCommercial 4.0 International License</a>.
