#pragma once

#include <string>
#include <chrono>
#include <glm/glm.hpp>

/**
 * Constructs a string of a memory consecutive float array with n elements.
 */
template<typename T>
std::string array_string(T* v, int n) {
    std::string out = "{";
    for(int i = 0; i < n ; i++) {
        if(i > 0)
            out += ", ";
        out += std::to_string(v[i]);
    }
    return out + "}";
}

std::string str(glm::vec2 v);
std::string str(glm::vec3 v);
std::string str(glm::vec4 v);
std::string str(glm::ivec2 v);
std::string str(glm::ivec3 v);
std::string str(glm::ivec4 v);
std::string str(glm::uvec2 v);
std::string str(glm::uvec3 v);
std::string str(glm::uvec4 v);
std::string str(glm::mat3 v);
std::string str(glm::mat4 v);



/**
 * Lightweight (but inaccurate) timer for measuring elapsed time in seconds using std::chrono::high_resolution_clock.
 * Usage:
 *
 * MiniTimer t;
 * // do stuff..
 * double seconds_since_creation = t.elapsed();
 */
class MiniTimer {
public:
    MiniTimer() : m_startTime(std::chrono::high_resolution_clock::now()) {}
    ~MiniTimer() = default;

    /**
     * Restarts the timer.
     * @return the time in seconds passed since the object was created or since the last restart.
     */
    double restart();

    /**
     * @return the time in seconds passed since the object was created or since the last time start was called.
     */
    double elapsed() {
        return static_cast<std::chrono::duration<double>>(std::chrono::high_resolution_clock::now() - m_startTime).count();
    }


private:
    std::chrono::time_point<std::chrono::system_clock> m_startTime;
};
