## GLSL Shader implementation for Compressed Segmentation Volume decompression

This directory contains exemplary shader files that can be used to decompress single bricks of a
CompressedSegmentationVolume with and without using rANS encoding and an optional separate frequency table for the finest
level-of-detail of bricks.

The header `csvol_constants.h` is defining constants that are used in the CPU and GPU implementation.
`compressed_segmentation_volume.glsl` contains all functions that are required to compress single bricks into arbitrary
locations within a uint Shader Storage Buffer Object. You will have to define multiple macros (see `compressed_segmentation_volume.glsl`) at
compile time and populate the encoding buffer with the compressed representation.

Assuming that the start locations of compressed bricks within the encoding array are given in g_brick_starts, a basic
implementation to use the shader code looks like this:

```
cache_idx = ...  // start index of the decompressed brick in the decoding array.
inv_lod = ...    // the inverse level-of-detail up to which the brick will be decoded (0 = coarsest).
brick_id = brick_to_1D(brick_index_3D, brick_count_3D);  

resetCSVolBrick(cache_idx, inv_lod);
decompressCSVolBrick(g_brick_starts[brick_id], g_brick_starts[brick_id+1], valid_brick_size, 0u, inv_lod, cache_idx * 8u);
```

Before the decompression can be carried out, all elements of the cache location must be set to the INVALID flag using
`resetCSVolBrick(..)`.
The brick in the cache must fit enough voxels of the given inerse level-of-detail (LoD).
The coarsest LoD should never be decoded as it can simply be read as the first palette entry from the encoding buffer. 
Given that the corasest LoD to decode is thus inverse LoD 1, at least 8 uints need to be available at cache_index.
The block starting at cache_idx in the decoding array must provide
* space for  8 uints when decompressing inverse LoD 1,
* space for 64 uints when decompressing inverse LoD 2,
* and so on, up to brick_size^3 uint for the finest possible inverse LoD.

After decompression, the decoding array contains the brick's original uint32 labels in morton order.
The function `readCSVolBrick` can be used to read from a decompressed brick at any given inverse LoD using a full
voxel index within brick_size^3.
